//////////////////////////////////////////////////////////////////////
// (C)2006 by Brian Postma
//////////////////////////////////////////////////////////////////////

#include "Demo.h"
#include "Menu8x8.h"
#include "Text8x8.h"
#include "Scroller.h"
#include "BouncingLogo.h"
#include "CopperBars.h"
#include "Stars.h"
#include "Soundmon.h"
#include "ShowWaves.h"
#include "DirRecurser.h"

#define KEYREPEATDELAY 40
#define KEYREPEATSPEED 15

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Demo::Demo()
{
  scroller=new Scroller(surface->w,200);
  txt=new Text8x8;
  menu=new Menu8x8(80,114,txt);
  BuildMenu();
  blogo=new BouncingLogo();
  copperbars=new CopperBars(0,79);
  stars=new Stars(81,193);
  waves=new ShowWaves(200);
  scrolldir=0;
  keydelay=0;

  modptr=LoadModule(menu->GetSelection());
  if (modptr!=NULL)
  {
    InitSoundmon(modptr);
  }
  showusage=false;
  volume=48;
  SetVolume(volume);
}

//////////////////////////////////////////////////////////////////////

Demo::~Demo()
{
  ShutdownSoundmon();
  UnloadModule(modptr);
  delete menu;
  delete txt;
  delete scroller;
  delete blogo;
  delete copperbars;
  delete stars;
  delete waves;
}

//////////////////////////////////////////////////////////////////////

void Demo::DoIt(SDL_Surface *surface)
{
  unsigned int ticks2,ticks1=SDL_GetTicks();

  ClrScreen(0);
  scroller->scroll(surface);
  copperbars->Draw(surface);
  stars->Draw(surface);
  menu->Draw(surface);
  blogo->Draw(surface);
  waves->Draw(surface);

  if (buttonvolplus || buttony)
  {
    buttonvolplus=false;
    buttony=false;
    volume+=4;
    if (volume>64) volume=64;
    SetVolume(volume);
  }
  if (buttonvolmin || buttonx)
  {
    buttonvolmin=false;
    buttonx=false;
    volume-=4;
    if (volume<0) volume=0;
    SetVolume(volume);
  }
  if (buttonb)
  {
    const char *ptr=menu->GetSelection();
    ShutdownSoundmon();
    UnloadModule(modptr);
    modptr=LoadModule(ptr);
    if (modptr!=NULL)
    {
      InitSoundmon(modptr);
    }
    buttonb=false;
  }
  if (up && scrolldir==0)
  {
    menu->PreviousSelection();
    keydelay=KEYREPEATDELAY;
    scrolldir=-1;
  }
  if (down && scrolldir==0)
  {
    menu->NextSelection();
    keydelay=KEYREPEATDELAY;
    scrolldir=1;
  }
  if (!up && !down)
  {
    scrolldir=0;
  }
  keydelay--;
  if (keydelay<=0)
  {
    keydelay=KEYREPEATSPEED;
    if (scrolldir>0)
      menu->NextSelection();
    else if (scrolldir<0)
      menu->PreviousSelection();
  }
  if (buttonl)
  {
    buttonl=false;
    menu->PreviousSelection8();
  }
  if (buttonr)
  {
    buttonr=false;
    menu->NextSelection8();
  }
  if (buttonselect)
  {
    showusage=!showusage;
    buttonselect=false;
  }
  ticks2=SDL_GetTicks();
  if (showusage)
    txt->DrawNumber(surface,(unsigned short)(5*(ticks2-ticks1)),280,184);
}

//////////////////////////////////////////////////////////////////////

void Demo::BuildMenu()
{
  DirRecurser dir("modules");
  string file,name;
  int i;
  bool rc=dir.First(file);
  while(rc)
  {
    name=file;
    i=name.find_first_of('/');
    if (i==string::npos)
      i=name.find_first_of('\\');
    if (i != name.npos)
      name.erase(0,i+1);
    if (name[0]!='.')
    {
      i=name.find_last_of('.');
      if (i!=name.npos)
        name.erase(i);
      menu->AddItem(name,file);
    }
    rc=dir.Next(file);
  }
}
