//////////////////////////////////////////////////////////////////////
// (C)2006 by Brian Postma
//////////////////////////////////////////////////////////////////////

#include "DirRecurser.h"
#ifdef WIN32
#include "windows.h"
#else
#include <dirent.h>
#include <sys/stat.h>
#include <sys/types.h>
#endif


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

DirRecurser::DirRecurser()
{
	dirlist.clear();
}


DirRecurser::DirRecurser(const char *dirname)
{
	Init(dirname);
}


DirRecurser::~DirRecurser()
{

}


void DirRecurser::Init(const char *dirname)
{
	dirlist.clear();
	it=0;
	RecurseDir(dirname);
}


void DirRecurser::RecurseDir(const char *dirname)
{
#ifdef WIN32
	WIN32_FIND_DATA findfiledata;
	HANDLE hfind;
	string tmp;
	int anyleft;

	tmp=(string)dirname+"/*";

	hfind = FindFirstFile(tmp.c_str(), &findfiledata);
	anyleft=(hfind!=INVALID_HANDLE_VALUE);
	while (anyleft)
	{
		if ((findfiledata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)==0)
		{
			tmp=(string)dirname+"/"+findfiledata.cFileName;
			dirlist.push_back(tmp);
		}
		else
		{
			if (strcmp(findfiledata.cFileName,"..")!=0 && strcmp(findfiledata.cFileName,".")!=0)
			{
				tmp=(string)dirname+"/"+findfiledata.cFileName;
				RecurseDir(tmp.c_str());
			}
		}
		anyleft=FindNextFile(hfind, &findfiledata);
	}
	FindClose(hfind);
#else
	struct dirent *entry;
	DIR *dirh;
	string tmp;
	struct stat sb;
	int rc;

	dirh=opendir(dirname);
	if (dirh==NULL)
		return;

	entry=readdir(dirh);
	while (entry!=NULL)
	{
		tmp=(string)dirname+'/'+entry->d_name;
		rc=stat(tmp.c_str(),&sb);
		if (rc==0 && ((sb.st_mode & S_IFREG) || (sb.st_mode & S_IFDIR)))
		{
			if (sb.st_mode & S_IFDIR)
			{
				if ( strcmp (entry->d_name,".")!=0 && 
				     strcmp (entry->d_name,"..")!=0 )
					RecurseDir(tmp.c_str());
			}
			else
			{
				dirlist.push_back(tmp);
			}
		}
		entry=readdir(dirh);
	}
	closedir(dirh);
#endif
}


bool DirRecurser::First(string &str)
{
	it=0;
	if (it<dirlist.size())
	{
		str=dirlist[it];
		return true;
	}
	else
	{
		str="";
		return false;
	}
}


bool DirRecurser::Next(string &str)
{
	it++;
	if (it<dirlist.size())
	{
		str=dirlist[it];
		return true;
	}
	else
	{
		str="";
		return false;
	}
}
