//////////////////////////////////////////////////////////////////////
// 
// File:        LSoundPlayer.cpp
// Author:      Brian Postma
// Created:     30 December 1999
//
// Purpose:     The hardware/OS dependent soundplayer layer. Provides
//              all means and methods to play sounds.
//
//////////////////////////////////////////////////////////////////////

#include "LSoundPlayer.h"

LSoundPlayer::LSoundPlayer(Module *modptr)
{
  this->modptr=modptr;
  playbuffer=NULL;
  nrofvoices=0;
  stereo=false;
  lastpos=0;
  mastervolume=50;
}


LSoundPlayer::~LSoundPlayer()
{
  CleanUp();
}


void LSoundPlayer::CleanUp()
{
  delete[] channels;
  delete[] playbuffer;
}


bool LSoundPlayer::Initialize(short nrofvoices, bool stereo)
{
  if (modptr==NULL)
    return false;

  this->nrofvoices=nrofvoices;

  if (stereo)
  {
    this->stereo=1;
  }
  else
  {
    this->stereo=0;
  }
  if ((channels=new LChannel[nrofvoices])==NULL)
    return false;
  if ((playbuffer=new short[BUFFERSIZE])==NULL)
    return false;
  for (int i=0; i<(BUFFERSIZE); i++)
    playbuffer[i]=0;

  return true;
}


bool LSoundPlayer::Play(short voicenr, BYTE instrument,
                        int frequency, int volume)
{
  WORD size, repeat, replen,dummy;
  BYTE *sampptr;

  if (instrument<1 || instrument>15 || voicenr>=nrofvoices || voicenr<0)
    return false;
  if (volume>64) volume=64;
  if (modptr->IsSynthInstrument(instrument))
  {
    modptr->GetSynthInstrument(instrument,size,sampptr);
    replen=size;
    repeat=0;
  }
  else
  {
    modptr->GetSampledInstrument(instrument,size,repeat,replen,dummy,sampptr);
  }
  channels[voicenr].SetPlayBuffer(sampptr,size,repeat,replen);
  channels[voicenr].SetVolume((volume*mastervolume)>>6);
  channels[voicenr].SetFrequency(frequency);
  channels[voicenr].StartPlay();
  return true;
}


bool LSoundPlayer::Stop(short voicenr)
{
  if (voicenr<0 || voicenr>=nrofvoices)
    return false;
  channels[voicenr].StopPlay();
  return true;
}


bool LSoundPlayer::SetPan(short voicenr, int val)
{
  if (voicenr<0 || voicenr>=nrofvoices)
    return false;
  if (val<0 || stereo==0) 
    channels[voicenr].SetLeftRight(true);
  else
    channels[voicenr].SetLeftRight(false);
  return true;
}


bool LSoundPlayer::SetFrequency(short voicenr, int frequency)
{
  if (voicenr<0 || voicenr>=nrofvoices)
    return false;
  channels[voicenr].SetFrequency(frequency);
  return true;
}


bool LSoundPlayer::SetVolume(short voicenr, int volume)
{
  if (voicenr<0 || voicenr>=nrofvoices)
    return false;
  if (volume>64) volume=64;
  channels[voicenr].SetVolume((volume*mastervolume)>>6);
  return true;
}


bool LSoundPlayer::Update(short voicenr, BYTE *memptr, 
                          int offset, int numofbytes)
{
  if (voicenr<0 || voicenr>=nrofvoices)
    return false;
  channels[voicenr].Update(memptr,offset,numofbytes);
  return true;
}


bool LSoundPlayer::Mix()
{
  int left,right;
  BYTE lefttmp, righttmp;
  short *cursor=playbuffer;
  int i,j;

  for (j=0; j<(BUFFERSIZE/2); j++)
  {
    left=0;
    right=0;
    for (i=0; i<nrofvoices; i++)
    {
      channels[i].GetNextBytes(lefttmp, righttmp);
      left+=lefttmp-128;
      right+=righttmp-128;
    }
    if (stereo)
    {
#ifdef _MAC
      *(cursor++)=left>>1;
      *(cursor++)=right>>1; 
#else
      *(cursor++)=left<<7;
      *(cursor++)=right<<7; 
#endif
    }
    else
    {
#ifdef _MAC
      *(cursor++)=(BYTE)left>>2;
      *(cursor++)=(BYTE)left>>2;
#else
      *(cursor++)=(BYTE)left<<6;
      *(cursor++)=(BYTE)left<<6;
#endif
    }
    channels[0].GetShowBuf(showbuf);
    channels[1].GetShowBuf(showbuf+32);
    channels[2].GetShowBuf(showbuf+64);
    channels[3].GetShowBuf(showbuf+96);

  }
  
  return true;
}
