//////////////////////////////////////////////////////////////////////
// (C)2006 by Brian Postma
//////////////////////////////////////////////////////////////////////

#include "gp2xkeys.h"
#include "SDLFramework.h"
#include <time.h>
#include <stdlib.h>

//////////////////////////////////////////////////////////////////////

const unsigned int SDLFramework::framedelay=19;
const int SDLFramework::scrwidth=320;
const int SDLFramework::scrheight=240;
const double SDLFramework::pi=3.141592653589793238462643383279502884197169;


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SDLFramework::SDLFramework()
{
  srand( (unsigned)time( NULL ) );
  mainsurface=NULL;
  joystick=NULL;
  quit=false; left=false; right=false; up=false; down=false;
  buttona=false; buttonb=false; buttonx=false; buttony=false;
  buttonl=false; buttonr=false; buttonvolplus=false; buttonvolmin=false;
  buttonselect=false;
#ifdef TARGET_GP2X
  SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK);
  joystick = SDL_JoystickOpen(0);
#else
  SDL_Init(SDL_INIT_VIDEO);
#endif
  SDL_ShowCursor(SDL_DISABLE);
  mainsurface = SDL_SetVideoMode(scrwidth, scrheight, 16, SDL_SWSURFACE);
  if (mainsurface==NULL)
    throw;
  surface=mainsurface;
}

//////////////////////////////////////////////////////////////////////

SDLFramework::~SDLFramework()
{
#ifdef TARGET_GP2X
  SDL_JoystickClose(joystick);
#endif
  SDL_FreeSurface(surface);
}

//////////////////////////////////////////////////////////////////////

bool SDLFramework::Init()
{
  return true;
}

//////////////////////////////////////////////////////////////////////

void SDLFramework::DoIt(SDL_Surface *surface)
{
}

//////////////////////////////////////////////////////////////////////

#ifdef TARGET_GP2X
void SDLFramework::KeyUp(const Uint8& key)
#else
void SDLFramework::KeyUp(const SDLKey& key)
#endif
{
  switch(key)
  {
    case GP2X_BUTTON_START:     quit=true; break;
    case GP2X_BUTTON_UP:        up=false; break;
    case GP2X_BUTTON_DOWN:      down=false; break;
    case GP2X_BUTTON_LEFT:      left=false; break;
    case GP2X_BUTTON_RIGHT:     right=false; break;
    case GP2X_BUTTON_UPLEFT:    up=false; left=false; break;
    case GP2X_BUTTON_UPRIGHT:   up=false; right=false; break; 
    case GP2X_BUTTON_DOWNLEFT:  down=false; left=false; break;
    case GP2X_BUTTON_DOWNRIGHT: down=false; right=false; break;
    case GP2X_BUTTON_A:         buttona=false; break;
    case GP2X_BUTTON_B:         buttonb=false; break;
    case GP2X_BUTTON_R:         buttonr=false; break;
    case GP2X_BUTTON_L:         buttonl=false; break;
    case GP2X_BUTTON_X:         buttonx=false; break;
    case GP2X_BUTTON_Y:         buttony=false; break;
    case GP2X_BUTTON_VOLPLUS:   buttonvolplus=false; break;
    case GP2X_BUTTON_VOLMIN:    buttonvolmin=false; break;
    case GP2X_BUTTON_SELECT:    buttonselect=false; break;
    default: break;
  }
}

//////////////////////////////////////////////////////////////////////

#ifdef TARGET_GP2X
void SDLFramework::KeyDown(const Uint8& key)
#else
void SDLFramework::KeyDown(const SDLKey& key)
#endif
{
  switch(key)
  {
    case GP2X_BUTTON_START:     quit=true; break;
    case GP2X_BUTTON_UP:        up=true; break;
    case GP2X_BUTTON_DOWN:      down=true; break;
    case GP2X_BUTTON_LEFT:      left=true; break;
    case GP2X_BUTTON_RIGHT:     right=true; break;
    case GP2X_BUTTON_UPLEFT:    up=true; left=true; break;
    case GP2X_BUTTON_UPRIGHT:   up=true; right=true; break;
    case GP2X_BUTTON_DOWNLEFT:  down=true; left=true; break;
    case GP2X_BUTTON_DOWNRIGHT: down=true; right=true; break;
    case GP2X_BUTTON_A:         buttona=true; break;
    case GP2X_BUTTON_B:         buttonb=true; break;
    case GP2X_BUTTON_R:         buttonr=true; break;
    case GP2X_BUTTON_L:         buttonl=true; break;
    case GP2X_BUTTON_X:         buttonx=true; break;
    case GP2X_BUTTON_Y:         buttony=true; break;
    case GP2X_BUTTON_VOLPLUS:   buttonvolplus=true; break;
    case GP2X_BUTTON_VOLMIN:    buttonvolmin=true; break;
    case GP2X_BUTTON_SELECT:    buttonselect=true; break;
    default: break;
  }
}

//////////////////////////////////////////////////////////////////////

void SDLFramework::KeyHandler()
{
  SDL_Event event;

  while(SDL_PollEvent(&event))
  {
    switch(event.type)
    {
#ifdef TARGET_GP2X
      case SDL_JOYBUTTONUP: KeyUp(event.jbutton.button); break;
      case SDL_JOYBUTTONDOWN: KeyDown(event.jbutton.button); break;
#else
      case SDL_KEYUP: KeyUp(event.key.keysym.sym); break;
      case SDL_KEYDOWN: KeyDown(event.key.keysym.sym); break;
      case SDL_QUIT: quit = true; break;
#endif
    }
  }
}

//////////////////////////////////////////////////////////////////////

void SDLFramework::run()
{
  if (!Init())
    return;
  Uint32 millis;

  surface=SDL_GetVideoSurface();
  screen=(short *)surface->pixels;

  while (!quit)
  {
    millis=SDL_GetTicks();
    KeyHandler();
    DoIt(surface);
#ifndef TARGET_GP2X
    while (SDL_GetTicks()-millis<framedelay);
#endif
    SDL_Flip(surface);
  }
}

//////////////////////////////////////////////////////////////////////

void SDLFramework::ClrScreen(short color)
{
  SDL_Rect rect;
  rect.x=0; rect.y=0; rect.w=scrwidth; rect.h=scrheight;
  SDL_FillRect(surface,&rect,color);
}

//////////////////////////////////////////////////////////////////////

void SDLFramework::PutPixel(SDL_Surface *surface, int x, int y,
                            unsigned short color)
{
  if (x<0 || x>=surface->w || y<0 || y>=surface->h) return;
  unsigned short *ptr=(unsigned short *)surface->pixels+x+y*surface->w;
  *ptr=color;
}

unsigned short SDLFramework::GetPixel(SDL_Surface *surface, 
                                      int x, int y)
{
  if (x<0 || x>=surface->w || y<0 || y>=surface->h) return 0;
  unsigned short *ptr=(unsigned short *)surface->pixels+x+y*surface->w;
  return *ptr;
}


//////////////////////////////////////////////////////////////////////

void SDLFramework::HLine(SDL_Surface *surface, int y, int x1, int x2, 
                         unsigned short color)
{
  if (y<0 || y>=surface->h) return;
  if (x2<x1) SwapInts(x1,x2);
  if (x2<0 || x1>=surface->w) return;
  if (x1<0) x1=0;
  if (x2>=surface->w) x2=surface->w-1;
  unsigned short *ptr=(unsigned short *)surface->pixels+x1+y*surface->w;
  for (int i=x1; i<=x2; i++)
  {  *ptr=color; ptr++; }

}

void SDLFramework::VLine(SDL_Surface *surface, int x, int y1, int y2, 
                         unsigned short color)
{
  int w=surface->w;
  if (x<0 || x>=w) return;
  if (y2<y1) SwapInts(y1,y2);
  if (y2<0 || y1>=surface->h) return;
  if (y1<0) y1=0;
  if (y2>=surface->h) y2=surface->h-1;
  unsigned short *ptr=(unsigned short*)surface->pixels+x+y1*w;
  for (int i=y1; i<=y2; i++)
  { *ptr=color; ptr+=w; };
}


void SDLFramework::Line(SDL_Surface *surface, int x1, int y1, int x2, int y2, 
                        unsigned short color)
{
  int deltax,deltay,delta;
  int count=1<<8,c=1<<16,d;
  unsigned short *ptr=(unsigned short *)surface->pixels;
  int w=surface->w;

  if (x1==x2)
  {
    VLine(surface,x1,y1,y2,color);
    return;
  }
  if (y1==y2)
  {
    HLine(surface,y1,x1,x2,color);
    return;
  }
  if (x1>x2)
  {
    deltax=x1-x2;
  }
  else
  {
    deltax=x2-x1;
  }
  if (y1>y2)
  {
    deltay=y1-y2;
  }
  else
  {
    deltay=y2-y1;
  }
  if (deltax>deltay)
  {
    if (x1>x2)
    {
      delta=x1; x1=x2; x2=delta;
      delta=y1; y1=y2; y2=delta;
    }   
    if (y1>y2)
    {
      d=-w;
    }
    else
    {
      d=w;
    }
    delta=(deltay<<16)/deltax;
    ptr+=x1+y1*w;
    for (int i=x1; i<=x2; i++)
    {
      *(ptr++)=color;
      count+=delta;
      if (count>=c)
      {
        ptr+=d;
        count-=c;
      }
    }
  }
  else
  {
    if (y1>y2)
    {
      delta=x1; x1=x2; x2=delta;
      delta=y1; y1=y2; y2=delta;
    }   
    if (x1>x2)
    {
      d=-1;
    }
    else
    {
      d=1;
    }
    delta=(deltax<<16)/deltay;
    ptr+=x1+y1*w;
    for (int i=y1; i<=y2; i++)
    {
      *ptr=color;
      ptr+=w;
      count+=delta;
      if (count>=(1<<16))
      {
        ptr+=d;
        count-=(1<<16);
      }
    }
  }
}

void SDLFramework::ClippedLine(SDL_Surface *surface, int x1, int y1, 
                               int x2, int y2, unsigned short color)
{
  int rc,x,y;
  int deltax, deltay;
  if (x1==x2)
  {
    VLine(surface,x1,y1,y2,color);
    return;
  }
  if (y1==y2)
  {
    HLine(surface,y1,x1,x2,color);
    return;
  }
  if (x1>x2)
  {
    deltax=x1-x2;
  }
  else
  {
    deltax=x2-x1;
  }
  if (y1>y2)
  {
    deltay=y1-y2;
  }
  else
  {
    deltay=y2-y1;
  }
  if (deltax>deltay)
  {
    if (x1>x2)
    {
      rc=x1; x1=x2; x2=rc;
      rc=y1; y1=y2; y2=rc;
    } 
    x=x1;
    y=y1<<16;
    rc=((y2-y1)<<16)/(x2-x1);
    for (int i=0; i<=deltax; i++)
    {
      PutPixel(surface,x,y>>16,color);
      x++; y+=rc;
    }
  }
  else
  {
    if (y1>y2)
    {
      rc=x1; x1=x2; x2=rc;
      rc=y1; y1=y2; y2=rc;
    }
    y=y1;
    x=x1<<16;
    rc=((x2-x1)<<16)/(y2-y1);
    for (int i=0; i<=deltay; i++)
    {
      PutPixel(surface,x>>16,y,color);
      y++; x+=rc;
    }

  }
}

//////////////////////////////////////////////////////////////////////

float SDLFramework::Random(float min, float max)
{
	return float(double(rand())/RAND_MAX)*(max-min)+min;
}

//////////////////////////////////////////////////////////////////////

void SDLFramework::FillTriangle(SDL_Surface *surface, int x1, int y1, 
                                int x2, int y2, int x3, int y3,
                                unsigned short color, 
                                unsigned short outlinecolor)
{
  // sort y coordinates
  if (y2>y3) { SwapInts(x2,x3); SwapInts(y2,y3); }
  if (y1>y2) { SwapInts(x1,x2); SwapInts(y1,y2); }
  if (y2>y3) { SwapInts(x2,x3); SwapInts(y2,y3); }
  int xx1, xx2, yy,dx1,dx2;

  if (y1==y2)
    HLine(surface,y1,x1,x2,color);
  else
  {
    dx1=((x2-x1)<<16)/(y2-y1);
    dx2=((x3-x1)<<16)/(y3-y1);
    yy=y1; xx1=x1<<16; xx2=x1<<16;
    while(yy<y2)
    {
      HLine(surface,yy,xx1>>16,xx2>>16,color);
      xx1+=dx1; xx2+=dx2; yy++;
    }
  }
  if (y2==y3)
    HLine(surface,y2,x2,x3,color);
  else
  {
    dx1=((x2-x3)<<16)/(y3-y2);
    dx2=((x1-x3)<<16)/(y3-y1);
    yy=y3; xx1=x3<<16; xx2=x3<<16;
    while(yy>=y2)
    {
      HLine(surface,yy,xx1>>16,xx2>>16,color);
      xx1+=dx1; xx2+=dx2; yy--;
    }
  }
  if (outlinecolor!=color)
  {
    ClippedLine(surface,x1,y1,x2,y2,outlinecolor);
    ClippedLine(surface,x2,y2,x3,y3,outlinecolor);
    ClippedLine(surface,x1,y1,x3,y3,outlinecolor);
  }
}

//////////////////////////////////////////////////////////////////////

