//////////////////////////////////////////////////////////////////////
// Integer based 3d plot/rotation etc. routines, 
//////////////////////////////////////////////////////////////////////
// (C)2006 by Brian Postma
//////////////////////////////////////////////////////////////////////

#include "SDLFramework3D.h"
#include <math.h>

// zoomfactor at 1024, this gives us a z range from -1000 to +30000
int SDLFramework3D::zoomfactor=1024;
short SDLFramework3D::zbuffer[320*240];
int SDLFramework3D::sinetable[SINETABLESIZE];

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SDLFramework3D::SDLFramework3D()
{
  for (int i=0; i<SINETABLESIZE; i++)
  {
    sinetable[i]=(int)((1<<16)*sin(2.0*double(i)*pi/double(SINETABLESIZE)));
  }
}

SDLFramework3D::~SDLFramework3D()
{

}

void SDLFramework3D::ClrScreen(short color)
{
  SDL_Rect rect;
  rect.x=0; rect.y=0; rect.w=scrwidth; rect.h=scrheight;
  SDL_FillRect(surface,&rect,color);
  for (int i=0; i<320*240; i++)
    zbuffer[i]=32767;
}

int SDLFramework3D::Sin(int angle)
{
  while(angle<0) angle+=SINETABLESIZE;
  while(angle>=SINETABLESIZE) angle-=SINETABLESIZE;
  return sinetable[angle];
}

int SDLFramework3D::Cos(int angle)
{
  return Sin(angle+(SINETABLESIZE>>2));
}

//////////////////////////////////////////////////////////////////////

void SDLFramework3D::Coord2D(SDL_Surface *surface, int x, int y, int z, 
                             int &x2d, int &y2d)
{
  if (z<=-zoomfactor)
  { x2d=-1; y2d=-1; return; }
  x2d=surface->w/2+(zoomfactor*x)/(z+zoomfactor);
  y2d=surface->h/2+(zoomfactor*y)/(z+zoomfactor);
}

//////////////////////////////////////////////////////////////////////

void SDLFramework3D::Fill3DTriangle(SDL_Surface *surface, 
                                    int *x, int *y, int *z,
                                    unsigned short color)
{
  int xx[3], yy[3];
  for (int i=0; i<3; i++)
  {
    Coord2D(surface,x[i],y[i],z[i],xx[i],yy[i]);
  }
  FillTriangle3D(surface,xx[0],yy[0],z[0],xx[1],yy[1],z[1],xx[2],yy[2],z[2],color);
}

//////////////////////////////////////////////////////////////////////

void SDLFramework3D::HLine3D(SDL_Surface *surface, int y, int x1, int x2, 
                           int z1, int z2, unsigned short color)
{
  if (y<0 || y>=surface->h) return;
  if (x2<x1) { SwapInts(x1,x2); SwapInts(z1,z2); }
  if (x2<0 || x1>=surface->w) return;
  int dz;
  if (x2==x1)
    dz=0;
  else
    dz=((z2-z1)<<16)/(x2-x1);
  int xmin=x1, xmax=x2;
  if (x1<0) xmin=0;
  if (x2>=surface->w) xmax=surface->w-1;
  unsigned short *ptr=(unsigned short *)surface->pixels+xmin+y*surface->w;
  short *zptr=zbuffer+xmin+y*320;
  int z;
  for (int i=xmin; i<=xmax; i++)
  { 
    z=z1+(((i-x1)*dz)>>16); 
    if (z<*zptr)
    {
      *ptr=color;
      *zptr=z;
    }
    ptr++;
    zptr++;
  }
}


//////////////////////////////////////////////////////////////////////

void SDLFramework3D::FillTriangle3D(SDL_Surface *surface,
                                  int x1, int y1, int z1,
                                  int x2, int y2, int z2,
                                  int x3, int y3, int z3,
                                  unsigned short color)
{
  // sort y coordinates
  if (y2>y3) { SwapInts(x2,x3); SwapInts(y2,y3); SwapInts(z2,z3); }
  if (y1>y2) { SwapInts(x1,x2); SwapInts(y1,y2); SwapInts(z1,z2); }
  if (y2>y3) { SwapInts(x2,x3); SwapInts(y2,y3); SwapInts(z2,z3); }
  int xx1,xx2,yy,dx1,dx2,dz1,dz2,zz1,zz2;

  if (y1==y2)
    HLine3D(surface,y1,x1,x2,z1,z2,color);
  else
  {
    dx1=((x2-x1)<<16)/(y2-y1);
    dx2=((x3-x1)<<16)/(y3-y1);
    dz1=((z2-z1)<<16)/(y2-y1);
    dz2=((z3-z1)<<16)/(y3-y1);
    yy=y1; xx1=x1<<16; xx2=x1<<16; zz1=z1<<16; zz2=z1<<16;
    while(yy<y2)
    {
      HLine3D(surface,yy,xx1>>16,xx2>>16,zz1>>16,zz2>>16,color);
      xx1+=dx1; xx2+=dx2; yy++; zz1+=dz1; zz2+=dz2;
    }
  }
  if (y2==y3)
    HLine3D(surface,y2,x2,x3,z2,z3,color);
  else
  {
    dx1=((x2-x3)<<16)/(y3-y2);
    dx2=((x1-x3)<<16)/(y3-y1);
    dz1=((z2-z3)<<16)/(y3-y2);
    dz2=((z1-z3)<<16)/(y3-y1);
    yy=y3; xx1=x3<<16; xx2=x3<<16; zz1=z3<<16; zz2=z3<<16;
    while(yy>=y2)
    {
      HLine3D(surface,yy,xx1>>16,xx2>>16,zz1>>16,zz2>>16,color);
      xx1+=dx1; xx2+=dx2; yy--; zz1+=dz1; zz2+=dz2;
    }
  }
}

//////////////////////////////////////////////////////////////////////

void SDLFramework3D::RotXclicks(int clicks, int *x, int *y, int *z, int count)
{
  int yy,zz;
  for (int i=0; i<count; i++)
  {
    zz=(int)(z[i]*Cos(clicks)+y[i]*Sin(clicks))>>16;
    yy=(int)(y[i]*Cos(clicks)-z[i]*Sin(clicks))>>16;
    z[i]=zz;
    y[i]=yy;
  }
}

void SDLFramework3D::RotYclicks(int clicks, int *x, int *y, int *z, int count)
{
  int xx,zz;
  for (int i=0; i<count; i++)
  {
    xx=(int)(x[i]*Cos(clicks)+z[i]*Sin(clicks))>>16;
    zz=(int)(z[i]*Cos(clicks)-x[i]*Sin(clicks))>>16;
    x[i]=xx;
    z[i]=zz;
  }
}

void SDLFramework3D::RotZclicks(int clicks, int *x, int *y, int *z, int count)
{
  int xx,yy;
  for (int i=0; i<count; i++)
  {
    xx=(int)(x[i]*Cos(clicks)+y[i]*Sin(clicks))>>16;
    yy=(int)(y[i]*Cos(clicks)-x[i]*Sin(clicks))>>16;
    x[i]=xx;
    y[i]=yy;
  }
}

void SDLFramework3D::RotXclicks(int clicks, int *x, int *y, int *z, int *xdst, int *ydst, int *zdst, int count)
{
  for (int i=0; i<count; i++)
  {
    zdst[i]=(int)(z[i]*Cos(clicks)+y[i]*Sin(clicks))>>16;
    ydst[i]=(int)(y[i]*Cos(clicks)-z[i]*Sin(clicks))>>16;
    xdst[i]=x[i];
  }
}

void SDLFramework3D::RotYclicks(int clicks, int *x, int *y, int *z, int *xdst, int *ydst, int *zdst, int count)
{
  for (int i=0; i<count; i++)
  {
    xdst[i]=(int)(x[i]*Cos(clicks)+z[i]*Sin(clicks))>>16;
    zdst[i]=(int)(z[i]*Cos(clicks)-x[i]*Sin(clicks))>>16;
    ydst[i]=y[i];
  }
}

void SDLFramework3D::RotZclicks(int clicks, int *x, int *y, int *z, int *xdst, int *ydst, int *zdst, int count)
{
  for (int i=0; i<count; i++)
  {
    xdst[i]=(int)(x[i]*Cos(clicks)+y[i]*Sin(clicks))>>16;
    ydst[i]=(int)(y[i]*Cos(clicks)-x[i]*Sin(clicks))>>16;
    zdst[i]=z[i];
  }
}

//////////////////////////////////////////////////////////////////////
// Floating point based rotations, slow on ARM machines, don't use
// unless really really necessary

void SDLFramework3D::RotX(double angle, int *x, int *y, int *z, int count)
{
  int yy,zz;
  for (int i=0; i<count; i++)
  {
    zz=(int)(z[i]*cos(angle)+y[i]*sin(angle));
    yy=(int)(y[i]*cos(angle)-z[i]*sin(angle));
    z[i]=zz;
    y[i]=yy;
  }
}


void SDLFramework3D::RotY(double angle, int *x, int *y, int *z, int count)
{
  int xx,zz;
  for (int i=0; i<count; i++)
  {
    xx=(int)(x[i]*cos(angle)+z[i]*sin(angle));
    zz=(int)(z[i]*cos(angle)-x[i]*sin(angle));
    x[i]=xx;
    z[i]=zz;
  }
}


void SDLFramework3D::RotZ(double angle, int *x, int *y, int *z, int count)
{
  int xx,yy;
  for (int i=0; i<count; i++)
  {
    xx=(int)(x[i]*cos(angle)+y[i]*sin(angle));
    yy=(int)(y[i]*cos(angle)-x[i]*sin(angle));
    x[i]=xx;
    y[i]=yy;
  }
}

//////////////////////////////////////////////////////////////////////

void SDLFramework3D::Translate(int t_x, int t_y, int t_z, 
                               int *x, int *y, int *z, int count)
{
  for (int i=0; i<count; i++)
  {
    x[i]+=t_x; y[i]+=t_y; z[i]+=t_z;
  }
}


