//////////////////////////////////////////////////////////////////////
// (C)2006 by Brian Postma
//////////////////////////////////////////////////////////////////////

#include "Soundmon.h"
#include "SDL.h"
#include "Player.h"
#include <fcntl.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#ifndef WIN32
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include <sys/ioctl.h>
#else
#include <io.h>
#endif
#ifdef TARGET_GP2X
#include <sys/soundcard.h>
#endif

Player *player=NULL;
Module *module=NULL;

int   buf_index=0;
bool stop=false;

#ifdef TARGET_GP2X
long soundfile=-1;
const char *sounddevice="/dev/dsp";
static pthread_t sound_thread_id;
#endif

void UnloadModule(unsigned char *memptr)
{
  delete[] memptr;
}

unsigned char *LoadModule(const char *filename)
{
  BYTE *memptr;
  int filehandle;
  if (filename==NULL)
    return NULL;
#ifndef WIN32
  struct stat filestat;
  if ((filehandle=open(filename,O_RDONLY))==-1)
    return NULL;
  fstat(filehandle,&filestat);
  if ((memptr=new unsigned char[filestat.st_size])==NULL)
  {
    close(filehandle);
    return NULL;
  }
  if ((read(filehandle,memptr,filestat.st_size))!=filestat.st_size)
  {
    close(filehandle);
    delete[] memptr;
    return NULL;
  }
  close(filehandle);
#else
  struct _stat filestat; 
  if ((filehandle=_open(filename,O_RDONLY|O_BINARY))==-1)
    return NULL;
  _fstat(filehandle,&filestat);
  if ((memptr=new unsigned char[filestat.st_size])==NULL)
  {
    _close(filehandle);
    return NULL;
  }
  if ((_read(filehandle,memptr,filestat.st_size))!=filestat.st_size)
  {
    _close(filehandle);
    delete[] memptr;
    return NULL;
  }
  _close(filehandle);
#endif
  return memptr;
}

#ifdef TARGET_GP2X
static void *sound_thread(void *dummy)
{
  short *ptr_src=(short *)player->GetSoundPlayer()->GetBuffer();

  while(!stop)
  {
    player->Mix();
    player->Run();
    write(soundfile,ptr_src,BUFFERSIZE*2);
    pthread_testcancel();
  }
  return 0;
}
#endif

#define BUFSIZE 1024

bool InitSoundmon(unsigned char *modptr)
{
#ifdef TARGET_GP2X
  if ((soundfile=open(sounddevice,O_WRONLY))==-1)
    return false;
  int rate=SAMPLERATE;
  int bits=AFMT_S16_LE;
  int stereo=1;
  int fragment=0x80007;
  bool rc=true;
  rc&=ioctl(soundfile, SNDCTL_DSP_SPEED, &rate)!=-1;
  rc&=ioctl(soundfile, SNDCTL_DSP_SETFMT, &bits)!=-1;
  rc&=ioctl(soundfile, SNDCTL_DSP_STEREO, &stereo)!=-1;
  rc&=ioctl(soundfile, SNDCTL_DSP_SETFRAGMENT, &fragment)!=-1;
  if (!rc)
  {
    close(soundfile);
    soundfile=-1;
    return false;
  }
#else
  if(SDL_Init(SDL_INIT_AUDIO)==-1)
    return false;
  SDL_AudioSpec spec;
  spec.freq=SAMPLERATE;
  spec.format=AUDIO_S16LSB;
  spec.channels=2;
  spec.samples=BUFSIZE;
  spec.callback=Soundmon_Callback;
  spec.userdata=NULL;
  if (SDL_OpenAudio(&spec, NULL)<0)
    return false;
#endif
  module=new Module(modptr);
  module->Init();
  player=new Player(module);
  player->Start();
  stop=false;
#ifdef TARGET_GP2X
  pthread_create( &sound_thread_id, NULL, sound_thread, NULL);
#else
  SDL_PauseAudio(0);
#endif
  return true;
}

void ShutdownSoundmon()
{
  stop=true;
#ifndef TARGET_GP2X
  SDL_CloseAudio();
#else
  pthread_join(sound_thread_id,NULL);
#endif
  player->Stop();
  delete player;
  player=NULL;
  delete module;
  module=NULL;
#ifdef TARGET_GP2X
  if (soundfile!=-1)
    close(soundfile);
  soundfile=-1;
#endif
}

void SetVolume(int volume)
{
  if (player==NULL)
    return;
  player->GetSoundPlayer()->SetMasterVolume(volume);
//  player->GetSoundPlayer()->SetVolume(0,volume);
//  player->GetSoundPlayer()->SetVolume(1,volume);
//  player->GetSoundPlayer()->SetVolume(2,volume);
//  player->GetSoundPlayer()->SetVolume(3,volume);
}


LSoundPlayer *GetSoundPlayer()
{
  if (player==NULL)
    return NULL;
  return player->GetSoundPlayer();
}


void Soundmon_Callback(void *userdata, unsigned char *buf, int req)
{
  int i;
  long *ptr_src, *ptr_dest;

  if (player==NULL || stop)
    return;
  ptr_src=(long *)player->GetSoundPlayer()->GetBuffer();
  ptr_dest=(long *)buf;

  i=0;
  while (i<(req/4))
  {
    if (buf_index==0)
    {
      player->Mix();
      player->Run();
    }
    ptr_dest[i]=ptr_src[buf_index];
    buf_index++;
    i++;
    if (buf_index>= (BUFFERSIZE/2) )
      buf_index=0;
  }
}

